import * as i0 from '@angular/core';
import { inject, ElementRef, NgZone, signal, EventEmitter, Directive, Input, Output, Component } from '@angular/core';
import { mergeDeep, naturalFieldComparator, buildChart, buildSingleSerieChart } from '@oneteme/jquery-core';
import { observeOn, asapScheduler, Subscription } from 'rxjs';
import ApexCharts from 'apexcharts';
import { fromPromise } from 'rxjs/internal/observable/innerFrom';
import * as i1 from '@angular/common';
import { CommonModule } from '@angular/common';

const ICONS = {
    previous: `<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 -960 960 960" style="width: 15px; height: 15px">
  <path d="M400-80 0-480l400-400 71 71-329 329 329 329-71 71Z"/>
  </svg>`,
    next: `<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 -960 960 960" style="width: 15px; height: 15px"><path d="m321-80-71-71 329-329-329-329 71-71 400 400L321-80Z"/></svg>`,
    pivot: `<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 -960 960 960" style="width: 15px; height: 15px"><path d="M400-640v-200h360q33 0 56.5 23.5T840-760v120H400ZM200-120q-33 0-56.5-23.5T120-200v-360h200v440H200Zm-80-520v-120q0-33 23.5-56.5T200-840h120v200H120ZM520-80 360-240l160-160 56 56-62 64h86q33 0 56.5-23.5T680-360v-88l-64 64-56-56 160-160 160 160-56 56-64-64v88q0 66-47 113t-113 47h-86l62 64-56 56Z"/></svg>`,
};

/**
 * Crée les boutons personnalisés pour la barre d'outils
 */
function customIcons(event, canPivot) {
    let customIcons = [
        {
            icon: ICONS.previous,
            title: 'Graphique précédent',
            class: 'custom-icon',
            click: function (chart, options, e) {
                event('previous');
            },
        },
        {
            icon: ICONS.next,
            title: 'Graphique suivant',
            class: 'custom-icon',
            click: function (chart, options, e) {
                event('next');
            },
        },
    ];
    if (canPivot) {
        customIcons.push({
            icon: ICONS.pivot,
            title: 'Pivot',
            class: 'custom-icon',
            click: function (chart, options, e) {
                event('pivot');
            },
        });
    }
    return customIcons;
}
function determineXAxisDataType(value) {
    if (value instanceof Date) {
        return 'datetime';
    }
    else if (typeof value === 'number') {
        return 'numeric';
    }
    else {
        return 'category';
    }
}
/**
 * Détermine le type d'axe X en fonction des données
 */
function getType(commonChart) {
    if (commonChart.series.length && commonChart.series[0].data.length) {
        if (commonChart.continue) {
            const x = commonChart.series[0].data[0].x;
            return determineXAxisDataType(x);
        }
        else {
            const categ = commonChart.categories[0];
            return determineXAxisDataType(categ);
        }
    }
    return 'datetime';
}
/**
 * Initialise les options de base communes à tous les graphiques
 */
function initCommonChartOptions(node, customEvent, ngZone, chartType, canPivot = true) {
    return {
        shouldRedraw: true,
        chart: {
            type: chartType,
            toolbar: {
                show: false,
                tools: {
                    download: false,
                    selection: false,
                    zoom: false,
                    zoomin: false,
                    zoomout: false,
                    pan: false,
                    reset: false,
                    customIcons: customIcons((arg) => {
                        ngZone.run(() => customEvent.emit(arg));
                    }, canPivot),
                },
            },
            events: {
                mouseMove: function () {
                    let toolbar = node.nativeElement.querySelector('.apexcharts-toolbar');
                    if (toolbar)
                        toolbar.style.visibility = 'visible';
                },
                mouseLeave: function () {
                    let toolbar = node.nativeElement.querySelector('.apexcharts-toolbar');
                    if (toolbar)
                        toolbar.style.visibility = 'hidden';
                },
            },
            zoom: {
                enabled: false,
            },
        },
        series: [],
        noData: {
            text: 'Aucune donnée',
        },
        xaxis: {},
        plotOptions: {},
    };
}
/**
 * Met à jour les options communes du graphique en fonction de la configuration fournie
 */
function updateCommonOptions(options, config) {
    const existingBarHorizontal = options?.plotOptions?.bar?.horizontal;
    const updatedOptions = mergeDeep(options, {
        chart: {
            height: config.height ?? '100%',
            width: config.width ?? '100%',
            stacked: config.stacked,
            toolbar: {
                show: config.showToolbar ?? false,
            },
        },
        title: {
            text: config.title,
        },
        subtitle: {
            text: config.subtitle,
        },
        xaxis: {
            title: {
                text: config.xtitle,
            },
        },
        yaxis: {
            title: {
                text: config.ytitle,
            },
        },
    }, config.options);
    const userSetHorizontal = config.options?.plotOptions?.bar?.horizontal !== undefined;
    if (existingBarHorizontal !== undefined && !userSetHorizontal) {
        if (!updatedOptions.plotOptions)
            updatedOptions.plotOptions = {};
        if (!updatedOptions.plotOptions.bar)
            updatedOptions.plotOptions.bar = {};
        updatedOptions.plotOptions.bar.horizontal = existingBarHorizontal;
    }
    return updatedOptions;
}
/**
 * Initialise un graphique ApexCharts de façon sécurisée
 */
function initChart(el, ngZone, options, chartInstance, subscription, debug) {
    if (debug) {
        console.log('Initialisation du graphique', { ...options });
    }
    // Exécution en dehors de la zone Angular pour de meilleures performances
    return ngZone.runOutsideAngular(() => {
        try {
            let chart = new ApexCharts(el.nativeElement, { ...options });
            chartInstance.set(chart);
            // Utilisation de fromPromise pour gérer le rendu asynchrone
            const renderSubscription = fromPromise(chart.render()
                .then(() => debug && console.log(new Date().getMilliseconds(), 'Rendu du graphique terminé'))
                .catch(error => {
                console.error('Erreur lors du rendu du graphique:', error);
                chartInstance.set(null);
            }))
                .pipe(observeOn(asapScheduler))
                .subscribe({
                next: () => debug && console.log(new Date().getMilliseconds(), 'Observable rendu terminé'),
                error: (error) => console.error('Erreur dans le flux Observable:', error)
            });
            // Ajout au gestionnaire d'abonnements
            subscription.add(renderSubscription);
            return chart;
        }
        catch (error) {
            console.error('Erreur lors de l\'initialisation du graphique:', error);
            return null;
        }
    });
}
/**
 * Met à jour les options du graphique de manière sécurisée
 */
function updateChartOptions(chartInstance, ngZone, options, redrawPaths = true, animate = true, updateSyncedCharts = false) {
    if (!chartInstance)
        return Promise.resolve();
    return ngZone.runOutsideAngular(() => chartInstance.updateOptions({ ...options }, redrawPaths, animate, updateSyncedCharts).catch(error => {
        console.error('Erreur lors de la mise à jour des options:', error);
        return Promise.resolve();
    }));
}
/**
 * Fonction commune pour gérer l'hydratation des graphiques
 */
function hydrateChart(changes, data, chartConfig, options, chartInstance, ngZone, updateDataFn, destroyFn, initFn, updateOptionsFn, debug) {
    if (debug)
        console.log('Hydratation du graphique', { ...changes });
    // Optimisation: regroupement des types de changements pour éviter les opérations redondantes
    const needsDataUpdate = changes['data'] || changes['config'] || changes['type'];
    const needsOptionsUpdate = Object.keys(changes).some(key => !['debug'].includes(key));
    // Mise à jour des données si nécessaire
    if (needsDataUpdate && data && chartConfig) {
        updateDataFn();
    }
    // Mise à jour spécifique pour isLoading
    if (changes['isLoading'] && chartInstance()) {
        options.noData.text = changes['isLoading'].currentValue
            ? 'Chargement des données...'
            : 'Aucune donnée';
        // Mise à jour immédiate des options de noData sans redessiner complètement
        updateChartOptions(chartInstance(), ngZone, {
            noData: options.noData
        }, false, false, false);
    }
    // Stratégie de mise à jour optimisée
    if (options.shouldRedraw) {
        if (debug)
            console.log('Recréation complète du graphique nécessaire', changes);
        destroyFn();
        initFn();
        delete options.shouldRedraw;
    }
    else if (needsOptionsUpdate) {
        if (debug)
            console.log('Mise à jour des options du graphique', changes);
        updateOptionsFn();
    }
}
/**
 * Fonction commune pour la destruction propre des graphiques
 */
function destroyChart(chartInstance, subscription) {
    // Désabonnement pour éviter les fuites de mémoire
    subscription.unsubscribe();
    // Nettoyage de l'instance du graphique
    if (chartInstance()) {
        chartInstance().destroy();
        chartInstance.set(null);
    }
}

class BarChartDirective {
    set isLoading(isLoading) {
        this._options.noData.text = isLoading
            ? 'Chargement des données...'
            : 'Aucune donnée';
    }
    set canPivot(canPivot) {
        this._canPivot = canPivot;
    }
    get canPivot() {
        return this._canPivot;
    }
    set config(config) {
        this._chartConfig = config;
        this._options = updateCommonOptions(this._options, config);
        this.configureTypeSpecificOptions();
    }
    constructor() {
        this.el = inject(ElementRef);
        this.ngZone = inject(NgZone);
        this.chartInstance = signal(null);
        this.subscription = new Subscription();
        this._canPivot = true;
        this.customEvent = new EventEmitter();
        this._options = initCommonChartOptions(this.el, this.customEvent, this.ngZone, 'bar');
    }
    init() {
        initChart(this.el, this.ngZone, this._options, this.chartInstance, this.subscription, this.debug);
    }
    ngOnChanges(changes) {
        if (this.debug)
            console.log(new Date().getMilliseconds(), 'Détection de changements', changes);
        if (changes['type'])
            this.updateType();
        this.ngZone.runOutsideAngular(() => {
            asapScheduler.schedule(() => this.hydrate(changes));
        });
    }
    ngOnDestroy() {
        destroyChart(this.chartInstance, this.subscription);
    }
    hydrate(changes) {
        hydrateChart(changes, this.data, this._chartConfig, this._options, this.chartInstance, this.ngZone, () => this.updateData(), () => this.ngOnDestroy(), () => this.init(), () => updateChartOptions(this.chartInstance(), this.ngZone, this._options), this.debug);
    }
    updateType() {
        this._options.chart.type = 'bar';
        this.configureTypeSpecificOptions();
        this._options.shouldRedraw = true;
    }
    configureTypeSpecificOptions() {
        if (!this._options.plotOptions)
            this._options.plotOptions = {};
        if (!this._options.plotOptions.bar)
            this._options.plotOptions.bar = {};
        if (this._options.plotOptions.bar.horizontal === undefined) {
            if (this.type === 'bar') {
                this._options.plotOptions.bar.horizontal = true;
            }
            else if (this.type === 'column') {
                this._options.plotOptions.bar.horizontal = false;
            }
        }
        if (this.type === 'funnel' || this.type === 'pyramid') {
            this._options.plotOptions.bar.isFunnel = true;
            this._options.plotOptions.bar.horizontal = true;
        }
    }
    updateData() {
        let sortedData = [...this.data];
        if (this.type == 'funnel') {
            sortedData = sortedData.sort(naturalFieldComparator('asc', this._chartConfig.series[0].data.y));
        }
        else if (this.type == 'pyramid') {
            sortedData = sortedData.sort(naturalFieldComparator('desc', this._chartConfig.series[0].data.y));
        }
        const commonChart = buildChart(sortedData, {
            ...this._chartConfig,
            pivot: !this.canPivot ? false : this._chartConfig.pivot,
        });
        this._options.series = commonChart.series.length
            ? commonChart.series.map((s) => ({
                data: s.data,
                name: s.name,
                color: s.color,
                group: s.stack,
            }))
            : [{ data: [] }];
        const newType = getType(commonChart);
        if (this._options.xaxis.type !== newType) {
            this._options.xaxis.type = newType;
            this._options.shouldRedraw = true;
        }
        this._options.xaxis.categories = commonChart.categories || [];
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: BarChartDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.2.12", type: BarChartDirective, isStandalone: true, selector: "[bar-chart]", inputs: { debug: "debug", type: "type", data: "data", isLoading: "isLoading", canPivot: "canPivot", config: "config" }, outputs: { customEvent: "customEvent" }, usesOnChanges: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: BarChartDirective, decorators: [{
            type: Directive,
            args: [{
                    standalone: true,
                    selector: '[bar-chart]',
                }]
        }], ctorParameters: function () { return []; }, propDecorators: { debug: [{
                type: Input
            }], type: [{
                type: Input,
                args: [{ required: true }]
            }], data: [{
                type: Input,
                args: [{ required: true }]
            }], customEvent: [{
                type: Output
            }], isLoading: [{
                type: Input
            }], canPivot: [{
                type: Input
            }], config: [{
                type: Input
            }] } });

class LineChartDirective {
    /**
     * Setter pour l'état de chargement
     */
    set isLoading(isLoading) {
        this._options.noData.text = isLoading
            ? 'Chargement des données...'
            : 'Aucune donnée';
    }
    /**
     * Setter pour la gestion du type
     */
    set type(type) {
        if (this._options.chart.type !== type) {
            this._options.chart.type = type;
            this._options.shouldRedraw = true;
        }
    }
    /**
     * Setter pour la configuration du graphique
     */
    set config(config) {
        this._chartConfig = config;
        this._options = updateCommonOptions(this._options, config);
        // Vérifier si sparkline est activé pour masquer l'axe Y
        if (config.options?.chart?.sparkline?.enabled) {
            if (!this._options.yaxis)
                this._options.yaxis = {};
            this._options.yaxis.show = false;
            this._options.yaxis.showAlways = false;
            if (!this._options.yaxis.labels)
                this._options.yaxis.labels = {};
            this._options.yaxis.labels.show = false;
        }
    }
    constructor() {
        this.el = inject(ElementRef);
        this.ngZone = inject(NgZone);
        this.chartInstance = signal(null);
        // Abonnement global pour éviter les fuites de mémoire
        this.subscription = new Subscription();
        this.customEvent = new EventEmitter();
        this._options = initCommonChartOptions(this.el, this.customEvent, this.ngZone, 'line');
    }
    /**
     * Initialise le graphique ApexCharts
     */
    init() {
        initChart(this.el, this.ngZone, this._options, this.chartInstance, this.subscription, this.debug);
    }
    /**
     * Gestion des changements de propriétés d'entrée
     */
    ngOnChanges(changes) {
        if (this.debug) {
            console.log(new Date().getMilliseconds(), 'Détection de changements', changes);
        }
        // Exécution en dehors de la zone Angular pour éviter les détections de changement inutiles
        this.ngZone.runOutsideAngular(() => {
            asapScheduler.schedule(() => this.hydrate(changes));
        });
    }
    /**
     * Nettoyage lors de la destruction du composant
     */
    ngOnDestroy() {
        destroyChart(this.chartInstance, this.subscription);
    }
    /**
     * Méthode centrale qui gère l'actualisation du graphique en fonction des changements
     * Optimisée pour minimiser les rendus inutiles
     */
    hydrate(changes) {
        hydrateChart(changes, this.data, this._chartConfig, this._options, this.chartInstance, this.ngZone, () => this.updateData(), () => this.ngOnDestroy(), () => this.init(), () => updateChartOptions(this.chartInstance(), this.ngZone, this._options), this.debug);
    }
    /**
     * Met à jour les données du graphique
     */
    updateData() {
        let commonChart = buildChart(this.data, { ...this._chartConfig, continue: true }, null);
        this._options.series = commonChart.series;
        // Détection de changement de type d'axe X qui nécessite une recréation
        let newType = getType(commonChart);
        if (this._options.xaxis.type != newType) {
            this._options.xaxis.type = newType;
            this._options.shouldRedraw = true;
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: LineChartDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.2.12", type: LineChartDirective, isStandalone: true, selector: "[line-chart]", inputs: { debug: "debug", data: "data", isLoading: "isLoading", type: "type", config: "config" }, outputs: { customEvent: "customEvent" }, usesOnChanges: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: LineChartDirective, decorators: [{
            type: Directive,
            args: [{
                    standalone: true,
                    selector: '[line-chart]',
                }]
        }], ctorParameters: function () { return []; }, propDecorators: { debug: [{
                type: Input
            }], data: [{
                type: Input,
                args: [{ required: true }]
            }], customEvent: [{
                type: Output
            }], isLoading: [{
                type: Input
            }], type: [{
                type: Input
            }], config: [{
                type: Input
            }] } });

class PieChartDirective {
    set isLoading(isLoading) {
        this._options.noData.text = isLoading
            ? 'Chargement des données...'
            : 'Aucune donnée';
    }
    set type(type) {
        this._type = type;
        if (this._options.chart.type !== type) {
            this._options.chart.type = type;
            this.configureTypeSpecificOptions();
            this._options.shouldRedraw = true;
        }
    }
    set config(config) {
        this._chartConfig = config;
        this._options = updateCommonOptions(this._options, config);
    }
    constructor() {
        this.el = inject(ElementRef);
        this.ngZone = inject(NgZone);
        this.chartInstance = signal(null);
        this.subscription = new Subscription();
        this._type = 'pie';
        this.customEvent = new EventEmitter();
        this._options = initCommonChartOptions(this.el, this.customEvent, this.ngZone, 'pie');
    }
    init() {
        initChart(this.el, this.ngZone, this._options, this.chartInstance, this.subscription, this.debug);
    }
    ngOnChanges(changes) {
        if (this.debug) {
            console.log(new Date().getMilliseconds(), 'Détection de changements', changes);
        }
        if (changes['type']) {
            this.updateType();
        }
        this.ngZone.runOutsideAngular(() => {
            asapScheduler.schedule(() => this.hydrate(changes));
        });
    }
    ngOnDestroy() {
        destroyChart(this.chartInstance, this.subscription);
    }
    hydrate(changes) {
        hydrateChart(changes, this.data, this._chartConfig, this._options, this.chartInstance, this.ngZone, () => this.updateData(), () => this.ngOnDestroy(), () => this.init(), () => updateChartOptions(this.chartInstance(), this.ngZone, this._options), this.debug);
    }
    updateType() {
        this.configureTypeSpecificOptions();
        this._options.shouldRedraw = true;
    }
    configureTypeSpecificOptions() {
        const currentType = this._type;
        if (currentType === 'pie') {
            this._options.chart.type = 'pie';
        }
        else if (currentType === 'donut') {
            this._options.chart.type = 'donut';
        }
        else if (currentType === 'radial') {
            this._options.chart.type = 'radialBar';
        }
        else if (currentType === 'polar') {
            this._options.chart.type = 'polarArea';
        }
        else if (currentType === 'radar') {
            this._options.chart.type = 'radar';
        }
    }
    updateData() {
        const chartConfig = { ...this._chartConfig, continue: false };
        const commonChart = this.data.length != 1 && this._type == 'radar'
            ? buildChart(this.data, chartConfig, null)
            : buildSingleSerieChart(this.data, chartConfig, null);
        if (this.data.length != 1 && this._type == 'radar') {
            this._options.series = commonChart.series;
        }
        else if (this._type == 'radar') {
            this._options.series = [
                {
                    name: 'Series 1',
                    data: commonChart.series.flatMap((s) => s.data.filter((d) => d != null)),
                },
            ];
        }
        else {
            this._options.series = commonChart.series.flatMap((s) => s.data.filter((d) => d != null));
        }
        this._options.labels = commonChart.categories || [];
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: PieChartDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.2.12", type: PieChartDirective, isStandalone: true, selector: "[pie-chart]", inputs: { debug: "debug", data: "data", isLoading: "isLoading", type: "type", config: "config" }, outputs: { customEvent: "customEvent" }, usesOnChanges: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: PieChartDirective, decorators: [{
            type: Directive,
            args: [{
                    standalone: true,
                    selector: '[pie-chart]',
                }]
        }], ctorParameters: function () { return []; }, propDecorators: { debug: [{
                type: Input
            }], data: [{
                type: Input,
                args: [{ required: true }]
            }], customEvent: [{
                type: Output
            }], isLoading: [{
                type: Input
            }], type: [{
                type: Input
            }], config: [{
                type: Input
            }] } });

class TreemapChartDirective {
    set isLoading(isLoading) {
        this._options.noData.text = isLoading
            ? 'Chargement des données...'
            : 'Aucune donnée';
    }
    set type(type) {
        this._type = type;
        if (this._options.chart.type !== type) {
            this._options.chart.type = type;
            this.configureTypeSpecificOptions();
            this._options.shouldRedraw = true;
        }
    }
    set config(config) {
        this._chartConfig = config;
        this._options = updateCommonOptions(this._options, config);
    }
    constructor() {
        this.el = inject(ElementRef);
        this.ngZone = inject(NgZone);
        this.chartInstance = signal(null);
        this.subscription = new Subscription();
        this._type = 'treemap';
        this.customEvent = new EventEmitter();
        this._options = initCommonChartOptions(this.el, this.customEvent, this.ngZone, 'treemap');
    }
    init() {
        initChart(this.el, this.ngZone, this._options, this.chartInstance, this.subscription, this.debug);
    }
    ngOnChanges(changes) {
        if (this.debug) {
            console.log(new Date().getMilliseconds(), 'Détection de changements', changes);
        }
        if (changes['type']) {
            this.updateType();
        }
        this.ngZone.runOutsideAngular(() => {
            asapScheduler.schedule(() => this.hydrate(changes));
        });
    }
    ngOnDestroy() {
        destroyChart(this.chartInstance, this.subscription);
    }
    hydrate(changes) {
        hydrateChart(changes, this.data, this._chartConfig, this._options, this.chartInstance, this.ngZone, () => this.updateData(), () => this.ngOnDestroy(), () => this.init(), () => updateChartOptions(this.chartInstance(), this.ngZone, this._options), this.debug);
    }
    updateType() {
        this.configureTypeSpecificOptions();
        this._options.shouldRedraw = true;
    }
    configureTypeSpecificOptions() {
        const currentType = this._type;
        if (currentType === 'treemap') {
            this._options.chart.type = 'treemap';
        }
        else if (currentType === 'heatmap') {
            this._options.chart.type = 'heatmap';
        }
    }
    updateData() {
        const commonChart = buildChart(this.data, { ...this._chartConfig, continue: true }, null);
        this._options.series = commonChart.series;
        const newType = getType(commonChart);
        if (this._options.xaxis.type != newType) {
            this._options.xaxis.type = newType;
            this._options.shouldRedraw = true;
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: TreemapChartDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.2.12", type: TreemapChartDirective, isStandalone: true, selector: "[treemap-chart]", inputs: { debug: "debug", data: "data", isLoading: "isLoading", type: "type", config: "config" }, outputs: { customEvent: "customEvent" }, usesOnChanges: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: TreemapChartDirective, decorators: [{
            type: Directive,
            args: [{
                    standalone: true,
                    selector: '[treemap-chart]',
                }]
        }], ctorParameters: function () { return []; }, propDecorators: { debug: [{
                type: Input
            }], data: [{
                type: Input,
                args: [{ required: true }]
            }], customEvent: [{
                type: Output
            }], isLoading: [{
                type: Input
            }], type: [{
                type: Input
            }], config: [{
                type: Input
            }] } });

class TestChartDirective {
    constructor() {
        this.el = inject(ElementRef);
    }
    ngAfterViewInit() {
        var options = {
            series: [{
                    name: 'Website Blog',
                    type: 'column',
                    data: [440, 505, 414, 671, 227, 413, 201, 352, 752, 320, 257, 160]
                }, {
                    name: 'Social Media',
                    type: 'area',
                    data: [23, 42, 35, 27, 43, 22, 17, 31, 22, 22, 12, 16]
                }],
            chart: {
                height: 350,
                type: 'area',
            },
            dataLabels: {
                enabled: true,
                enabledOnSeries: [1]
            },
            labels: ['01 Jan 2001', '02 Jan 2001', '03 Jan 2001', '04 Jan 2001', '05 Jan 2001', '06 Jan 2001', '07 Jan 2001', '08 Jan 2001', '09 Jan 2001', '10 Jan 2001', '11 Jan 2001', '12 Jan 2001'],
            xaxis: {
                type: 'datetime'
            },
            yaxis: [{
                    title: {
                        text: 'Website Blog',
                    },
                }, {
                    opposite: true,
                    title: {
                        text: 'Social Media'
                    }
                }]
        };
        this._chart = new ApexCharts(this.el.nativeElement, options);
        this._chart.render();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: TestChartDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.2.12", type: TestChartDirective, isStandalone: true, selector: "[test-chart]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: TestChartDirective, decorators: [{
            type: Directive,
            args: [{
                    standalone: true,
                    selector: '[test-chart]'
                }]
        }] });

class RangeChartDirective {
    set isLoading(isLoading) {
        this._options.noData.text = isLoading
            ? 'Chargement des données...'
            : 'Aucune donnée';
    }
    set canPivot(canPivot) {
        this._canPivot = canPivot;
    }
    get canPivot() {
        return this._canPivot;
    }
    set type(type) {
        this._type = type;
        if (this._options.chart.type !== type) {
            this._options.chart.type = type;
            this.configureTypeSpecificOptions();
            this._options.shouldRedraw = true;
        }
    }
    set config(config) {
        this._chartConfig = config;
        this._options = updateCommonOptions(this._options, config);
    }
    constructor() {
        this.el = inject(ElementRef);
        this.ngZone = inject(NgZone);
        this.chartInstance = signal(null);
        this.subscription = new Subscription();
        this._type = 'rangeArea';
        this._canPivot = true;
        this.customEvent = new EventEmitter();
        this._options = initCommonChartOptions(this.el, this.customEvent, this.ngZone, 'rangeArea');
    }
    init() {
        initChart(this.el, this.ngZone, this._options, this.chartInstance, this.subscription, this.debug);
    }
    ngOnChanges(changes) {
        if (this.debug) {
            console.log(new Date().getMilliseconds(), 'Détection de changements', changes);
        }
        if (changes['type'])
            this.updateType();
        this.ngZone.runOutsideAngular(() => {
            asapScheduler.schedule(() => this.hydrate(changes));
        });
    }
    ngOnDestroy() {
        destroyChart(this.chartInstance, this.subscription);
    }
    hydrate(changes) {
        hydrateChart(changes, this.data, this._chartConfig, this._options, this.chartInstance, this.ngZone, () => this.updateData(), () => this.ngOnDestroy(), () => this.init(), () => updateChartOptions(this.chartInstance(), this.ngZone, this._options), this.debug);
    }
    updateType() {
        this.configureTypeSpecificOptions();
        this._options.shouldRedraw = true;
    }
    configureTypeSpecificOptions() {
        const currentType = this._type;
        if (currentType === 'rangeBar') {
            if (!this._options.plotOptions)
                this._options.plotOptions = {};
            if (!this._options.plotOptions.bar)
                this._options.plotOptions.bar = {};
            this._options.plotOptions.bar.horizontal = true;
        }
        else if (currentType === 'rangeColumn') {
            this._options.chart.type = 'rangeBar';
            if (!this._options.plotOptions)
                this._options.plotOptions = {};
            if (!this._options.plotOptions.bar)
                this._options.plotOptions.bar = {};
            this._options.plotOptions.bar.horizontal = false;
        }
    }
    updateData() {
        let commonChart = buildChart(this.data, {
            ...this._chartConfig,
            continue: true,
            pivot: !this.canPivot ? false : this._chartConfig.pivot,
        }, null);
        this._options.series = commonChart.series;
        let newType = getType(commonChart);
        if (this._options.xaxis.type != newType) {
            this._options.xaxis.type = newType;
            this._options.shouldRedraw = true;
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: RangeChartDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.2.12", type: RangeChartDirective, isStandalone: true, selector: "[range-chart]", inputs: { debug: "debug", data: "data", isLoading: "isLoading", canPivot: "canPivot", type: "type", config: "config" }, outputs: { customEvent: "customEvent" }, usesOnChanges: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: RangeChartDirective, decorators: [{
            type: Directive,
            args: [{
                    standalone: true,
                    selector: '[range-chart]',
                }]
        }], ctorParameters: function () { return []; }, propDecorators: { debug: [{
                type: Input
            }], data: [{
                type: Input,
                args: [{ required: true }]
            }], customEvent: [{
                type: Output
            }], isLoading: [{
                type: Input
            }], canPivot: [{
                type: Input
            }], type: [{
                type: Input
            }], config: [{
                type: Input
            }] } });

class ChartComponent {
    constructor() {
        this._charts = {
            pie: { possibleType: ['pie', 'donut', 'polar', 'radar', 'radial'] },
            donut: { possibleType: ['pie', 'donut', 'polar', 'radar', 'radial'] },
            polar: { possibleType: ['pie', 'donut', 'polar', 'radar', 'radial'] },
            radar: { possibleType: ['pie', 'donut', 'polar', 'radar', 'radial'] },
            radial: { possibleType: ['pie', 'donut', 'polar', 'radar', 'radial'] },
            line: { possibleType: ['line', 'area'] },
            area: { possibleType: ['line', 'area'] },
            bar: { possibleType: ['bar', 'column', 'heatmap', 'treemap'] },
            column: { possibleType: ['bar', 'column', 'heatmap', 'treemap'] },
            heatmap: { possibleType: ['bar', 'column', 'heatmap', 'treemap'] },
            treemap: { possibleType: ['bar', 'column', 'heatmap', 'treemap'] },
            funnel: { possibleType: ['funnel', 'pyramid'], canPivot: false },
            pyramid: { possibleType: ['funnel', 'pyramid'], canPivot: false },
            rangeArea: { possibleType: ['rangeArea', 'rangeBar', 'rangeColumn'], canPivot: false },
            rangeBar: { possibleType: ['rangeArea', 'rangeBar', 'rangeColumn'] },
            rangeColumn: { possibleType: ['rangeArea', 'rangeBar', 'rangeColumn'] },
        };
    }
    set value(type) {
        this._type = type;
    }
    change(event) {
        let charts = this._charts[this._type].possibleType;
        let indexOf = charts.indexOf(this._type);
        if (indexOf != -1) {
            if (event == 'previous') {
                this._type =
                    indexOf == 0 ? charts[charts.length - 1] : charts[indexOf - 1];
                return;
            }
            if (event == 'next') {
                this._type =
                    indexOf == charts.length - 1 ? charts[0] : charts[indexOf + 1];
                return;
            }
        }
        if (event == 'pivot') {
            this.config = this.config.pivot
                ? { ...this.config, pivot: false }
                : { ...this.config, pivot: true };
            return;
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: ChartComponent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "16.2.12", type: ChartComponent, isStandalone: true, selector: "chart", inputs: { value: ["type", "value"], config: "config", data: "data", isLoading: "isLoading", debug: "debug" }, ngImport: i0, template: "<div line-chart *ngIf=\"_type == 'line' || _type == 'area'\" [type]=\"_type\" [debug]=\"debug\" [config]=\"config\" [data]=\"data\" [isLoading]=\"isLoading\" (customEvent)=\"change($event)\"></div>\n<div pie-chart *ngIf=\"_type == 'donut' || _type == 'polar' || _type == 'radial' || _type == 'radar' || _type == 'pie'\" [type]=\"_type\" [config]=\"config\" [data]=\"data\" [isLoading]=\"isLoading\" (customEvent)=\"change($event)\"></div>\n<div bar-chart *ngIf=\"_type == 'bar' || _type == 'column' || _type == 'funnel' || _type == 'pyramid'\" [type]=\"_type\" [config]=\"config\" [data]=\"data\" [isLoading]=\"isLoading\" (customEvent)=\"change($event)\" [canPivot]=\"_charts[_type].canPivot\"></div>\n<div treemap-chart *ngIf=\"_type == 'treemap' || _type == 'heatmap'\" [type]=\"_type\" [config]=\"config\" [data]=\"data\" [isLoading]=\"isLoading\" (customEvent)=\"change($event)\"></div>\n<div range-chart *ngIf=\"_type == 'rangeArea' || _type == 'rangeBar' || _type == 'rangeColumn'\" [type]=\"_type\" [config]=\"config\" [data]=\"data\" [isLoading]=\"isLoading\" (customEvent)=\"change($event)\" [canPivot]=\"_charts[_type].canPivot\"></div>\n", dependencies: [{ kind: "ngmodule", type: CommonModule }, { kind: "directive", type: i1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: PieChartDirective, selector: "[pie-chart]", inputs: ["debug", "data", "isLoading", "type", "config"], outputs: ["customEvent"] }, { kind: "directive", type: BarChartDirective, selector: "[bar-chart]", inputs: ["debug", "type", "data", "isLoading", "canPivot", "config"], outputs: ["customEvent"] }, { kind: "directive", type: LineChartDirective, selector: "[line-chart]", inputs: ["debug", "data", "isLoading", "type", "config"], outputs: ["customEvent"] }, { kind: "directive", type: RangeChartDirective, selector: "[range-chart]", inputs: ["debug", "data", "isLoading", "canPivot", "type", "config"], outputs: ["customEvent"] }, { kind: "directive", type: TreemapChartDirective, selector: "[treemap-chart]", inputs: ["debug", "data", "isLoading", "type", "config"], outputs: ["customEvent"] }] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.2.12", ngImport: i0, type: ChartComponent, decorators: [{
            type: Component,
            args: [{ standalone: true, imports: [CommonModule, PieChartDirective, BarChartDirective, LineChartDirective, RangeChartDirective, TreemapChartDirective], selector: 'chart', template: "<div line-chart *ngIf=\"_type == 'line' || _type == 'area'\" [type]=\"_type\" [debug]=\"debug\" [config]=\"config\" [data]=\"data\" [isLoading]=\"isLoading\" (customEvent)=\"change($event)\"></div>\n<div pie-chart *ngIf=\"_type == 'donut' || _type == 'polar' || _type == 'radial' || _type == 'radar' || _type == 'pie'\" [type]=\"_type\" [config]=\"config\" [data]=\"data\" [isLoading]=\"isLoading\" (customEvent)=\"change($event)\"></div>\n<div bar-chart *ngIf=\"_type == 'bar' || _type == 'column' || _type == 'funnel' || _type == 'pyramid'\" [type]=\"_type\" [config]=\"config\" [data]=\"data\" [isLoading]=\"isLoading\" (customEvent)=\"change($event)\" [canPivot]=\"_charts[_type].canPivot\"></div>\n<div treemap-chart *ngIf=\"_type == 'treemap' || _type == 'heatmap'\" [type]=\"_type\" [config]=\"config\" [data]=\"data\" [isLoading]=\"isLoading\" (customEvent)=\"change($event)\"></div>\n<div range-chart *ngIf=\"_type == 'rangeArea' || _type == 'rangeBar' || _type == 'rangeColumn'\" [type]=\"_type\" [config]=\"config\" [data]=\"data\" [isLoading]=\"isLoading\" (customEvent)=\"change($event)\" [canPivot]=\"_charts[_type].canPivot\"></div>\n" }]
        }], propDecorators: { value: [{
                type: Input,
                args: [{ alias: 'type', required: true }]
            }], config: [{
                type: Input,
                args: [{ required: true }]
            }], data: [{
                type: Input,
                args: [{ required: true }]
            }], isLoading: [{
                type: Input
            }], debug: [{
                type: Input
            }] } });

/*
 * Public API Surface of jquery-apexcharts
 */

/**
 * Generated bundle index. Do not edit.
 */

export { BarChartDirective, ChartComponent, LineChartDirective, PieChartDirective, RangeChartDirective, TestChartDirective, TreemapChartDirective };
//# sourceMappingURL=oneteme-jquery-apexcharts.mjs.map
